/* Global Includes */
#include <stdio.h>
#include <stdlib.h>
#include "cdefs.h"

#ifdef _UNIX
#  define vect_maxfunc    vect_maxfunc_               
#  define vect_maxfunc_sf vect_maxfunc_sf_                  
#  define vect_maxfunc_cf vect_maxfunc_cf_                  
#endif

typedef struct { real_4 re; real_4 im; } cmplx_8;


void vect_maxfunc_sf(real_4 *sA, real_4 *sB, real_4 *sC, int_4 *n, 
                     real_4 *sV, int_4 *sI, int_4 *sImin, int_4 *sImax) 
{
  real_4 *svA, *svB, *svC;
  real_4 svMAXVAL; 
  int_4  svMAXIDX;
  int_4  i,iminidx,imaxidx;

  /* Use local references and decrement for 1-based indexing */
  svA = (real_4 *) sA;  
  svA--;
  svB = (real_4 *) sB;
  svB--;
  svC = (real_4 *) sC;
  svC--;

  iminidx = *sImin; 
  imaxidx = *sImax; 
  svMAXVAL = svA[iminidx];
  svMAXIDX = iminidx;      

  for (i=1; i<=*n; i++) {
    svC[i] = (svA[i] > svB[i]) ? (svA[i]) : (svB[i]);
    if (svC[i] >= svMAXVAL) {
      if ((i >= iminidx) && (i <= imaxidx)){
        svMAXVAL = svC[i];
        svMAXIDX = i;
      }
    }
  }

  *sV = svC[svMAXIDX];
  *sI = svMAXIDX;

  return; 
}


void vect_maxfunc_cf(cmplx_8 *sA, cmplx_8 *sB, cmplx_8 *sC, int_4 *n, 
                     cmplx_8 *sV, int_4 *sI, int_4 *sImin, int_4 *sImax) 
{
  cmplx_8 *svA, *svB, *svC;
  real_4 svMAXVAL;
  int_4  svMAXIDX;
  real_4 powA,powB;
  int_4  i,iminidx,imaxidx;

  /* Use local references and decrement for 1-based indexing */
  svA = (cmplx_8 *) sA;  
  svA--;
  svB = (cmplx_8 *) sB;
  svB--;
  svC = (cmplx_8 *) sC;
  svC--;

  iminidx = *sImin; 
  imaxidx = *sImax; 
  powA = (svA[1].re * svA[1].re) + (svA[1].im * svA[1].im);
  svMAXVAL = powA;        
  svMAXIDX = iminidx;      

  for (i=1; i<=*n; i++) {
    powA = (svA[i].re * svA[i].re) + (svA[i].im * svA[i].im);
    powB = (svB[i].re * svB[i].re) + (svB[i].im * svB[i].im);
    svC[i] = (powA > powB) ? (svA[i]) : (svB[i]);
    if ((powA >= svMAXVAL) || (powB >= svMAXVAL)) {
      if ((i >= iminidx) && (i <= imaxidx)){
        svMAXVAL = (powA > powB) ? (powA) : (powB);
        svMAXIDX = i;
      }
    }
  }

  *sV = svC[svMAXIDX];
  *sI = svMAXIDX;

  return; 
}
