/********************************************************************/
/*<EXP>*
SDDSSPLIT - Splits SDDS packet data into type 1000 data file/pipe 
            and type 3000 packet header file/pipe
            
<INPUT>   Input SDDS packetized data (Type 1000 or 2000)
<DATA>    (Optional) Output data file (Type 1000)
<HEADERS> (Optional) Output packet header file (Type 3000)
<PAUSEFILE> (Optional) Input file, when it exists pause processing
                       (should be full pathname/file, i.e. /tmp/junk - 
                        note this works only on Unix platforms)

Output data xdelta = input xdelta, so make sure input xdelta is
1 / (# of data samples/sec) instead of 1 / (# of data samples + SDDS
header bytes /sec).

SWITCHES:
       /DEBUG	- debug info level 
       /TL=     - input transfer length in SDDS packets
       /RAW     - Raw mode implies 64 bytes for SDDS header instead of
                  normal 56 (56 transmitted over sdds) 
                  (what you would get w/ UOPT flag from ICE acquisition)
       /ICE     - ICE header instead of sdds header
       /ICEDS=n - ICE data size in type 1000 elements (=/TL for sourcpic
                  if acquired via this)

REVISION       PROGRAMMER      DESCRIPTION
--------       ----------      -------------------------------------
16-Sep-2005    dkc             baseline
24-Oct-2005    dkc             fixed various bugs
01-May-2006    dkc             PAUSEFILE functionality
30-Jan-2007    dkc             Support for SI,CB,CI added
05-Apr-2007    dkc             Fixed SI,CB,CI support for ICE data

/*<EXP>*/
/********************************************************************/


// Includes
#include <primitive.h>
#include <fintrinsics.h>
#include <math.h> 
#include "hwpackets.h"
#include <sys/stat.h>

// Begin: Main Routine
void mainroutine () 
{

  // Variable Declarations
  CPHEADER HIN,HDATA,HHEADERS;

  int_1  *data=NULL;

  int_4  sw_debug;                      //Debug output switch
  int_4  sw_tl;                         //Transfer length switch
  int_4  sw_raw;
  int_4  sw_ice;
  int_4  sw_iceds;
  
  int_4  hdrstart,datastart;
  int_4  hdrsize,pktsize,datasize,offset;
  int_4  i,ngot;
  
  // PAUSEFILE processing
  string pausefile;
  char pausefile_char[256];
  int_4 pausetime;  
  struct stat pausefile_buffer;
  
  // Get input
    m_initialize(HIN);
      HIN.file_name = m_apick(1);
      HIN.format_allow = "SB,SI,CB,CI";
      HIN.type_allow   = "1000,2000";
      m_inok(HIN);
      m_force1000(HIN);  
  
  // Get command line switches
    sw_debug = m_get_switch("DEBUG");

    sw_ice   = m_get_switch("ICE");
    if (sw_ice > 0) {
      sw_iceds = m_get_switch("ICEDS");
      if (sw_iceds <= 0) m_error("Must specify /ICEDS with /ICE\n");
      hdrsize  = icehdrsize;
      datasize = sw_iceds * HIN.bpa; // in bytes
    }
    else {
      sw_raw   = m_get_switch("RAW");
      if (sw_raw > 0) hdrsize = rawhdrsize;
      else            hdrsize = sddshdrsize;
      datasize = sddsdatasize;   // in bytes
    }    
    pktsize = hdrsize + datasize;
    
    sw_tl    = m_get_switch_def("TL",32);
    sw_tl    = sw_tl * pktsize;
    data = (int_1 *)malloc((size_t)(sw_tl));      

    pausefile = m_apick(4);
    strcpy(pausefile_char,pausefile.c_str());
    
    if (sw_debug > 0) {
      printf("------ SDDSSPLIT debug ------ \n");
      printf("Packet size  = %d bytes \n",pktsize);
      printf("Header size  = %d bytes \n",hdrsize);
      printf("Data size    = %d bytes \n",datasize);
      printf("Transfer len = %d bytes \n",sw_tl);
    }    
      
  // Setup output files
    m_propagate(HIN,HDATA);
      HDATA.file_name = m_apick(2);
      HDATA.type = 1000;
      HDATA.switches = "OPTIONAL";
      m_allocate(HDATA);
      
    m_initialize(HHEADERS);
      HHEADERS.file_name = m_apick(3);
      HHEADERS.type = 3000;
      HHEADERS.format = "NH";
      i = 0;
      offset = 0;
      if (sw_ice > 0) {
        HHEADERS.subr[i].name   = "BSY1";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 1;
        HHEADERS.subr[i].name   = "BSY2";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 1;
        HHEADERS.subr[i].name   = "BSY3";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 1;
        HHEADERS.subr[i].name   = "PTYP";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 1;
        HHEADERS.subr[i].name   = "COUN";
        HHEADERS.subr[i].format = "SL";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 4;
        HHEADERS.subr[i].name   = "ELEM";
        HHEADERS.subr[i].format = "SL";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 4;
        HHEADERS.subr[i].name   = "USER";
        HHEADERS.subr[i].format = "SI";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 2;
        HHEADERS.subr[i].name   = "CHAN";
        HHEADERS.subr[i].format = "SI";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 2;
        HHEADERS.subr[i].name   = "ID1";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 1;
        HHEADERS.subr[i].name   = "ID2";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 1;
        HHEADERS.subr[i].name   = "REP";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 1;
        HHEADERS.subr[i].name   = "BPA";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 1;
        HHEADERS.subr[i].name   = "MODE";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 1;
        HHEADERS.subr[i].name   = "TYPE";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 1;
        HHEADERS.subr[i].name   = "TCMO";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 1;
        HHEADERS.subr[i].name   = "TSCT";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 1;
        HHEADERS.subr[i].name   = "TCOFF";
        HHEADERS.subr[i].format = "SD";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 8;
        HHEADERS.subr[i].name   = "TCWS";
        HHEADERS.subr[i].format = "SD";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 8;
        HHEADERS.subr[i].name   = "TCFS";
        HHEADERS.subr[i].format = "SD";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 8;
        HHEADERS.subr[i].name   = "USR1";
        HHEADERS.subr[i].format = "SL";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 4;
        HHEADERS.subr[i].name   = "USR2";
        HHEADERS.subr[i].format = "SL";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 4;
        HHEADERS.subr[i].name   = "USR3";
        HHEADERS.subr[i].format = "SL";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 4;
        HHEADERS.subr[i].name   = "USR4";
        HHEADERS.subr[i].format = "SL";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 4;
      }
      else {
        if (sw_raw > 0) {
           HHEADERS.subr[i].name   = "KEYS";
           HHEADERS.subr[i].format = "SI";
           HHEADERS.subr[i].offset = offset;
           i++; offset += 2;
           HHEADERS.subr[i].name   = "PORT";
           HHEADERS.subr[i].format = "SI";
           HHEADERS.subr[i].offset = offset;
           i++; offset += 2;
           HHEADERS.subr[i].name   = "ADDR";
           HHEADERS.subr[i].format = "SL";
           HHEADERS.subr[i].offset = offset;
           i++; offset += 4;
        }
        HHEADERS.subr[i].name   = "FOID";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 1;
        HHEADERS.subr[i].name   = "BITS";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 1;
        HHEADERS.subr[i].name   = "FRAM";
        HHEADERS.subr[i].format = "SI";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 2;
        HHEADERS.subr[i].name   = "TCIN";
        HHEADERS.subr[i].format = "SI";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 2;
        HHEADERS.subr[i].name   = "TCCO";
        HHEADERS.subr[i].format = "SI";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 2;
        HHEADERS.subr[i].name = "TCTI";
        HHEADERS.subr[i].format = "8B";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 8;
        HHEADERS.subr[i].name = "TCEX";
        HHEADERS.subr[i].format = "SL";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 4;
        HHEADERS.subr[i].name = "SSCI";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 12;
        HHEADERS.subr[i].name = "SSDI";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 4;
        HHEADERS.subr[i].name = "AADI";
        HHEADERS.subr[i].format = "SB";
        HHEADERS.subr[i].offset = offset;
        i++; offset += 20;
      }  
      if (offset != hdrsize) printf("DKC coding error, offset %d != hdrsize %d\n",offset,hdrsize);
      HHEADERS.subrecords = i;
      HHEADERS.record_length = hdrsize;
      HHEADERS.switches = "OPTIONAL";
      m_allocate(HHEADERS);    

  // Signal dataflow ready for processing block
    m_sync();

    while (m_do(sw_tl/HIN.bpa, HIN.xfer_len)) {
    
       pausetime = stat(pausefile_char,&pausefile_buffer);
       if (pausetime == 0) {m_pause(0.1);continue;}  // Wait until file removed
    
       m_grabx(HIN,data,ngot);
       if (sw_debug > 1) printf("ngot = %d, pktsize = %d \n",ngot,pktsize);
       ngot = ngot * HIN.bpa/ pktsize;
       for (i=0;i<ngot;i++) {
         hdrstart  = i * pktsize;
         if (HHEADERS.open) m_filad(HHEADERS,&data[hdrstart],1);
         datastart = hdrstart + hdrsize;
         if (sw_ice < 1 && HIN.format[1] == 'I') {
            // For raw sdds data - convert from Big Endian to little Endian
            m_convert("IEEE",&data[datastart],'I',datasize / 2);
                //  divide by 2 since datasize in bytes and we have int data
         }
         if (HDATA.open)    m_filad(HDATA,&data[datastart],datasize / HIN.bpa);
       }
          
    }
        
  // Close up open files/pipes
    if (data != NULL) free(data);
    m_close(HIN);
    m_close(HDATA);
    m_close(HHEADERS);

} // End: Main Routine
