/* Create Wrappers */
#include "cdefs.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "native.h"
#include "libserial.c"

#define CMETHOD(M) Serial_## M
#define JMETHOD(M) Java_nxm_gigexd_lib_Serial_## M

#define PTR (void*)
#define CPTR (void*)(long)
#define JPTR (jlong)(long)

JNIEXPORT jlong JNICALL JMETHOD (alloc) (JNIEnv *env, jobject obj) {
  void *SP;
  int_4 rv = serialport_create(&SP);
  return JPTR SP;
}

JNIEXPORT jint JNICALL JMETHOD (sendCommand) (JNIEnv *env, jobject obj, jlong handle, jstring str) {
  LIBSERIAL_Object *SP = (LIBSERIAL_Object *) CPTR handle;
  char *cstr = (char*)GetString(str);
  char c_tx[MAX_SERIALIO_STRINGLEN];
  int_4 i;

  // Initialize "C" strings
  c_tx[0] = 0x00;

  // Terminate the input TX string according to serial device specs
  strncpy(c_tx, cstr, MAX_SERIALIO_STRINGLEN);
  i = strnlen(c_tx, MAX_SERIALIO_STRINGLEN);
  i = (i < (MAX_SERIALIO_STRINGLEN-2)) ? (i) : (MAX_SERIALIO_STRINGLEN-2);
  switch (SP->carriage) {
    case (1):
      c_tx[i]   = 0x0d;  // <CR> only
      c_tx[i+1] = 0x00;  // NULL       
      break;
    case (2):
      c_tx[i]   = 0x0a;  // <LF> only
      c_tx[i+1] = 0x00;  // NULL       
      break;
    case (3):
      c_tx[i]   = 0x0d;  // <CR>
      c_tx[i+1] = 0x0a;  // <LF> 
      c_tx[i+2] = 0x00;  // NULL       
      break;
    case (4):
      c_tx[i]   = 0x0a;  // <LF>
      c_tx[i+1] = 0x0d;  // <CR> 
      c_tx[i+2] = 0x00;  // NULL       
      break;
    default:
      c_tx[i]   = 0x0d;  // <CR>
      c_tx[i+1] = 0x0a;  // <LF> 
      c_tx[i+2] = '\0';  // NULL       
  }

  // Send the command and check for status
  int_4 rv = serialport_write(SP, c_tx);

  ReleaseString(str, cstr);
  return (jint) rv;
}

JNIEXPORT jint JNICALL JMETHOD (setDelay) (JNIEnv *env, jobject obj, jlong handle, jint num) {
  serialport_set_iodelay(CPTR handle, (int_4) num);
  return 0;
}

JNIEXPORT jint JNICALL JMETHOD (setCarriage) (JNIEnv *env, jobject obj, jlong handle, jint num) {
  serialport_set_carriage(CPTR handle, (int_4) num);
  return 0;
}

JNIEXPORT jint JNICALL JMETHOD (setDebug) (JNIEnv *env, jobject obj, jlong handle, jint num) {
  serialport_set_debuglevel (CPTR handle, (int_4) num);
  return 0;
}

JNIEXPORT jint JNICALL JMETHOD (setDevice) (JNIEnv *env, jobject obj, jlong handle, jstring str) {
  char *cstr = (char*)GetString(str);
  int_4 stat = serialport_init(CPTR handle, cstr);
  ReleaseString(str, cstr);
  return (jint) stat;
}

JNIEXPORT jstring JNICALL JMETHOD(getStatus) (JNIEnv *env, jobject obj, jlong handle) {
  //char mystr[1024] = {"hello, tom"};
  char c_rx[MAX_SERIALIO_STRINGLEN];
  // Initialize "C" strings
  c_rx[0] = 0x00;
  serialport_read(CPTR handle, c_rx, MAX_SERIALIO_STRINGLEN);
  return (*env)->NewStringUTF(env, c_rx);
}

JNIEXPORT jint  JNICALL JMETHOD(free) (JNIEnv *env, jobject obj, jlong handle) {
  // Release serial port and destroy handle
  void *SP = CPTR handle;
  serialport_destroy(&SP);
  return 0;
}

